from __future__ import with_statement

import pyglet
pyglet.options['debug_gl'] = False
#from pyglet.gl import *
from gletools.gl import *
pyglet.resource.path = ['data']
pyglet.resource.reindex()
import math
import random
import gletools
from config import *

images = {}
def getimage( file ):
	if file not in images:
		i = pyglet.resource.image( file )
		i.anchor_x = i.width / 2
		i.anchor_y = i.height / 2
		try:
			t = i.get_mipmapped_texture()
		except:
			pass
		else:
			i.get_texture = i.get_mipmapped_texture
		images[file] = i
	return images[file]

def getsprite( image, **kwargs ):
	s = pyglet.sprite.Sprite( getimage( image ), **kwargs )
	return s

DELAYS = {
	'arrow.mp3': 0.25,
	'bubble-pop.mp3': 0.25,
	'icarus-flap.mp3': 0.25,
	'monster-drop.mp3': 0.5,
	'submersion.mp3': 0.25,
}

def getplayer():
		player = pyglet.media.Player()
		player.eos_action = player.EOS_PAUSE
		player.volume = SOUNDEFFECTS
		return player

def getsound( soundfile ):
	return pyglet.resource.media( soundfile )

def playsound( soundfile ):
	if SOUNDEFFECTS:
		player = getplayer()
		player.queue( getsound( soundfile ) )
		if soundfile in DELAYS:
			player.seek( DELAYS[soundfile] )
		player.play()

class Controller( object ):
	def draw( self, *args ):
		pass
	def think( self, *args ):
		pass
	def on_mouse_motion( self, *args ):
		pass
	def on_mouse_press( self, *args ):
		pass
	def on_mouse_release( self, *args ):
		pass
	def on_mouse_drag( self, *args ):
		pass
	def on_key_press( self, *args ):
		pass
	def on_key_release( self, *args ):
		pass

class Cloud( object ):
	def __init__( self, width ):
		self.x = random.random() * width
		self.y = self.randomheight()
		self.parallax = 0.1 + random.random() * 0.4
		self.sprite = getsprite( 'cloud%d.png'%random.randint( 1, 3 ) )
	def randomheight( self ):
		return 200 + random.random() * 300
	def draw( self ):
		with gletools.Matrix:
			glTranslatef( self.x - controller.camx * self.parallax, self.y, 0 )
			self.sprite.draw()
	def think( self ):
		if self.x - controller.camx * self.parallax + self.sprite.width / 2 < 0:
			self.y = self.randomheight()
			self.sprite = getsprite( 'cloud%d.png'%random.randint( 1, 3 ) )
			self.x = controller.camx * self.parallax + self.sprite.width / 2 + WIDTH
		elif self.x - controller.camx * self.parallax - self.sprite.width / 2 > WIDTH:
			self.y = self.randomheight()
			self.sprite = getsprite( 'cloud%d.png'%random.randint( 1, 3 ) )
			self.x = controller.camx * self.parallax - self.sprite.width / 2

class Icarus( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
		self.sprites = [getsprite( 'icarus%d.png'%(i+1) ) for i in range( 5 )]
		self.sprite = 0
		self.vx, self.vy = 0, 0
		self.rot = 0
		self.animdelay = 0
		self.hits = []
		self.trophies = []
	def draw( self ):
		glRotatef( self.rot, 0, 0, 1 )
		with gletools.Matrix:
			glScalef( 0.8, 0.8, 1 )
			self.sprites[self.sprite].draw()
		for hit in self.hits:
			with gletools.Matrix:
				glRotatef( -hit.unrot, 0, 0, 1 )
				glTranslatef( hit.x - self.x, hit.y - self.y, 0 )
				hit.draw()
	def draw2( self ):
		glLoadIdentity()
		glTranslatef( 50, 50, 0 )
		for trophy in self.trophies:
			glTranslatef( 80, 0, 0 )
			with gletools.Matrix:
				glScalef( 0.2, 0.2, 0 )
				trophy.draw()
	def insun( self ):
		return self.y > HEIGHT - 100
	def inwater( self ):
		return self.y <= 100
	def falling( self ):
		return pyglet.window.key.DOWN in keys or self.insun() or len( self.hits ) >= 5 or self.inwater()
	def animate( self ):
		self.animdelay += 1
		if self.animdelay == 5:
			self.animdelay = 0
			if (self.sprite == 0 or self.sprite == 1) and self.falling():
				self.sprite = 1
			elif self.sprite != 0:
				self.sprite += 1
				self.sprite %= len( self.sprites )
	def gethit( self, other ):
		if isinstance( other, Blocker ):
			playsound( 'arrow.mp3' )
			vx = self.x - other.x
			vy = self.y - other.y
			l = math.sqrt( vx ** 2 + vy ** 2 )
			self.vx += 5 * vx / l
			self.vy += 3 * vy / l
	def think( self ):
		self.animate()
		if pyglet.window.key.LEFT in keys and not self.inwater():
			self.rot -= (-20 + self.rot) * 0.1
		elif pyglet.window.key.RIGHT in keys and not self.inwater():
			self.rot += (-70 - self.rot) * 0.1
		else:
			self.rot -= self.rot * 0.02
		self.vy -= controller.gravity
		v = math.sqrt( self.vx**2 + self.vy**2 )
		radrot = -self.rot * math.pi / 180
		rx, ry = math.sin( radrot ), math.cos( radrot )
		vx, vy = self.vx, self.vy
		drag = rx * vx + ry * vy
		if (drag < 0) != self.falling():
			# drag only going down normally, drag only going up when DOWN pressed
			self.vx -= 0.2 * drag * rx
			self.vy -= 0.2 * drag * ry
		self.x += self.vx
		self.y += self.vy
		for hit in self.hits:
			hit.x += self.vx
			hit.y += self.vy
		camspeed = 0.0004
		if controller.time < 30 * FPS:
			camspeed *= float( controller.time ) / (30 * FPS)
		if self.x - controller.camx > 500:
			controller.camx += float( self.x - controller.camx - 500 )**2 * camspeed
		if self.x - controller.camx < 200:
			controller.camx -= float( 200 - (self.x - controller.camx) )**2 * camspeed
		
		if self.y < 250 and self.animdelay == 0 and random.random() < 0.3:
			controller.objects.insert( 0, Tentacle( self.x, 20 ) )
		if self.insun():
			for hit in self.hits:
				obj = SimpleObject( hit.x, hit.y, vx = self.vx, vy = self.vy, gravity = controller.gravity )
				def hitdraw( hit = hit ):
					hit.draw()
				obj.draw = hitdraw
				addobj( obj )
			self.hits = []
		if self.inwater():
			if not getattr( self, 'gameoversoundplayed', False ):
				playsound( 'submersion.mp3' )
				self.gameoversoundplayed = True
			gameover = 'Press SPACE to restart!'
			if controller.message.text != gameover:
				controller.message.text = gameover
		elif len( self.trophies ) >= MIN_TROPHIES and self.x >= MIN_X and controller.time >= MIN_TIME:
			congratulations = 'You have reached Sicily!'
			if controller.message.text != congratulations:
				controller.message.text = congratulations

		self.collide()
	def collide( self ):
		def dist2( obj ):
			return (obj.x - self.x) ** 2 + (obj.y - self.y) ** 2
		for obj in controller.objects:
			if hasattr( obj, 'gethit' ) and dist2( obj ) < 30 ** 2:
				hit = obj.gethit( self )
				if hit:
					hit.unrot = self.rot
					self.hits.append( hit )
	def on_key_press( self, symbol, modifiers ):
		if symbol == pyglet.window.key.UP and self.sprite == 0 and not self.inwater():
			playsound( 'icarus-flap.mp3' )
			self.sprite = 1
			self.animdelay = 0
			radrot = -self.rot * math.pi / 180
			rx, ry = math.sin( radrot ), math.cos( radrot )
			self.vx += 5 * rx
			self.vy += 5 * ry
		elif symbol == pyglet.window.key.DOWN and self.sprite == 0:
			self.sprite = 1
		elif symbol == pyglet.window.key.SPACE and self.inwater():
			global controller
			controller = Playing()
#	def on_key_release( self, symbol, modifiers ):
#		if symbol == pyglet.window.key.DOWN:
#			self.sprite = 0
#			self.on_key_press( pyglet.window.key.UP, modifiers )

class Blocker( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
		self.vx, self.vy = 0, 0
		self.rot = 0
		self.hits = []
		self.smalldelay = 0
	def draw( self ):
		glRotatef( self.rot, 0, 0, 1 )
		with gletools.Matrix:
			self.sprite.draw()
		for hit in self.hits:
			with gletools.Matrix:
				glRotatef( -hit.unrot, 0, 0, 1 )
				glTranslatef( hit.x - self.x, hit.y - self.y, 0 )
				hit.draw()
	def falling( self ):
		return len( self.hits ) >= self.life
	def think( self ):
		removeifoutside( self )
		if self not in controller.objects and len( controller.icarus.trophies ) < MIN_TROPHIES:
			if controller.camx - self.sprite.width / 2 < self.x < controller.camx + WIDTH + self.sprite.width / 2:
				controller.icarus.trophies.append( self )
		if self.falling():
			if not getattr( self, 'soundplayed', False ):
				playsound( 'monster-drop.mp3' )
				self.soundplayed = True
			self.vy -= controller.gravity
		self.x += self.vx
		self.y += self.vy
		for hit in self.hits:
			hit.x += self.vx
			hit.y += self.vy
		
		if not self.falling():
			self.collide()

		self.smalldelay += 1
		if self.smalldelay == self.smallinterval:
			self.smalldelay = 0
			addobj( self.newsmall() )
	def collide( self ):
		def dist2( obj ):
			return (obj.x - self.x) ** 2 + (obj.y - self.y) ** 2
		for obj in controller.objects:
			if self.canbehurtby( obj ) and hasattr( obj, 'gethit' ) and dist2( obj ) < self.radius ** 2:
				self.hit()
				hit = obj.gethit( self )
				if hit:
					hit.unrot = self.rot
					self.hits.append( hit )

class Minotaur( Blocker ):
	def __init__( self, x, y ):
		Blocker.__init__( self, x, y )
		self.sprite = getsprite( 'minotaur.png' )
		self.life = 3
		self.radius = 120
		self.vrot = 0
		self.smallinterval = 10 * FPS
	def newsmall( self ):
		return SmallMinotaur( self.x, self.y )
	def canbehurtby( self, obj ):
		return not isinstance( obj, SmallMinotaur )
	def think( self ):
		Blocker.think( self )
		self.rot += self.vrot
		self.vrot -= self.rot * 0.1
		self.vrot *= 0.9
	def hit( self ):
		self.vrot = -2

class Squid( Blocker ):
	def __init__( self, x, y ):
		Blocker.__init__( self, x, y )
		self.sprite = getsprite( 'squid.png' )
		self.life = 3
		self.radius = 120
		self.vrot = 0
		self.smallinterval = 5 * FPS
	def newsmall( self ):
		return SmallSquid( self.x - 50 + random.randint( 0, 1 ) * 100, self.y - 50 )
	def canbehurtby( self, obj ):
		return not isinstance( obj, SmallSquid )
	def think( self ):
		Blocker.think( self )
		self.rot += self.vrot
		self.vrot -= self.rot * 0.1
		self.vrot *= 0.9
	def hit( self ):
		self.vrot = -2

class Medusa( Blocker ):
	def __init__( self, x, y ):
		Blocker.__init__( self, x, y )
		self.sprite = getsprite( 'medusa.png' )
		self.life = 3
		self.radius = 100
		self.vrot = 0
		self.smallinterval = 2.5 * FPS
	def newsmall( self ):
		return SmallMedusa( self.x - 30 + random.randint( 0, 1 ) * 68, self.y + 35 )
	def canbehurtby( self, obj ):
		return not isinstance( obj, SmallMedusa )
	def think( self ):
		Blocker.think( self )
		self.rot += self.vrot
		self.vrot -= self.rot * 0.1
		self.vrot *= 0.9
	def hit( self ):
		self.vrot = -2

class SmallEnemy( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
		self.sprite = getsprite( 'medusa-small.png' )
		self.vx, self.vy = 0, 0
		self.jetdelay = 0
		self.tentacles_can_grab = True
	def jet( self ):
		self.jetdelay = 0
		vx = controller.icarus.x - self.x
		vy = controller.icarus.y - self.y
		l = math.sqrt( vx ** 2 + vy ** 2 )
		vx *= self.jetspeed / l
		vy *= self.jetspeed / l
		self.vx = vx
		self.vy = vy
		rot = -90 + math.atan2( vy, vx ) * 180 / math.pi
		self.makejet( rot )
	def makejet( self, rot ):
		addobj( Jet( self.x, self.y, rot ) )
	def gethit( self, target ):
		playsound( 'hit.mp3' )
		removeobj( self )
		return self
	def draw( self ):
		self.sprite.draw()
	def draw2( self ):
		self.sprite.draw()
	def think( self ):
		removeifoutside( self )
		self.vy -= controller.gravity
		self.x += self.vx
		self.y += self.vy
		self.jetdelay += 1
		if self.jetdelay == self.jetinterval:
			self.jet()

class SmallMedusa( SmallEnemy ):
	def __init__( self, x, y ):
		SmallEnemy.__init__( self, x, y )
		self.sprite = getsprite( 'medusa-small.png' )
		self.jetspeed = 4
		self.jetinterval = FPS

class SmallMinotaur( SmallEnemy ):
	def __init__( self, x, y ):
		SmallEnemy.__init__( self, x, y )
		self.sprite = getsprite( 'minotaur-small.png' )
		self.jetspeed = 10
		self.jetinterval = FPS*2

class SmallSquid( SmallEnemy ):
	def __init__( self, x, y ):
		SmallEnemy.__init__( self, x, y )
		self.sprite = getsprite( 'squid-small.png' )
		self.jetspeed = 3
		self.jetinterval = FPS * 0.5
	def makejet( self, rot ):
		jet = Jet( self.x, self.y, rot )
		jet.sprites = [getsprite( 'blackjet%d.png'%(i+1) ) for i in range( 4 )]
		addobj( jet )

class Tentacle( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
		self.bubbles = [[0, 0, getsprite( 'bubble%d.png'%random.randint( 1, 3 ), blend_dest = GL_ONE )] for i in range( 3 )]
		self.tentacle = getsprite( 'tentacle%d.png'%random.randint( 1, 3 ) )
		self.sprite = self.bubbles
		self.vy = 3
		self.hits = []
	def draw( self ):
		if self.sprite == self.bubbles:
			for x, y, sprite in self.bubbles:
				with gletools.Matrix:
					glScalef( 0.5, 0.5, 1 )
					glTranslatef( x, y, 0 )
					sprite.draw()
		else:
			self.sprite.draw()
			if self.vy <= 0:
				self.drawhits()
	def draw2( self ):
		if self.vy > 0:
			self.drawhits()
	def drawhits( self ):
		for hit in self.hits:
			with gletools.Matrix:
				glTranslatef( hit.x - self.x, hit.y - self.y, 0 )
				hit.draw()
	def think( self ):
		self.y += self.vy
		if self.sprite == self.bubbles:
			for b in self.bubbles:
				b[0] += random.random() * 4 - 2
				b[1] += random.random() * 4 - 2
			if self.y > 200:
#				playsound( 'bubble-pop.mp3' )
#				playsound( 'tentacle.mp3' )
				self.y = -60
				self.sprite = self.tentacle
				self.vy = 7
		else:
			if self.y < -100:
				removeobj( self )
			for hit in self.hits:
				hit.y += self.vy
			self.collide()
			self.vy -= controller.gravity
	def collide( self ):
		def dist2( obj ):
			return (obj.x - self.x) ** 2 + (0.25 *(obj.y - self.y)) ** 2
		for obj in controller.objects:
			if getattr( obj, 'tentacles_can_grab', False ) and dist2( obj ) < 25 ** 2:
				self.hits.append( obj )
				removeobj( obj )
			if isinstance( obj, Icarus ) and dist2( obj ) < 15 ** 2 and not getattr( obj, 'already_grabbed', False ):
				obj.already_grabbed = True
				self.hits.append( obj )
				obj.inwater = lambda: True

class SimpleObject( object ):
	def __init__( self, x, y, sprite = None, scale = 1, rot = 0, vx = 0, vy = 0, gravity = 0 ):
		self.x, self.y = x, y
		if sprite:
			self.sprite = getsprite( sprite )
		self.scale = scale
		self.rot = rot
		self.vx = vx
		self.vy = vy
		self.gravity = gravity
	def draw( self ):
		glScalef( self.scale, self.scale, 1 )
		glRotatef( self.rot, 0, 0, 1 )
		self.sprite.draw()
	def think( self ):
		removeifoutside( self )
		self.vy -= self.gravity
		self.x += self.vx
		self.y += self.vy

class Jet( object ):
	def __init__( self, x, y, rot = 0 ):
		self.x, self.y = x, y
		self.sprites = [getsprite( 'jet%d.png'%(i+1) ) for i in range( 4 )]
		self.sprite = 0
		self.animdelay = 0
		self.rot = rot
	def draw( self ):
		glRotatef( self.rot, 0, 0, 1 )
		self.sprites[self.sprite].draw()
	def think( self ):
		self.animdelay += 1
		if self.animdelay == 3:
			self.animdelay = 0
			self.sprite += 1
			if self.sprite == len( self.sprites ):
				self.sprite -= 1
				removeobj( self )

class Water( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
		self.sprite = getsprite( 'water.png' )
		self.rot = 0
		self.vrot = 1
	def draw( self ):
#		glScalef( 0.8, 0.8, 1 )
		glRotatef( self.rot, 0, 0, 1 )
		self.sprite.draw()
	def think( self ):
		removeifoutside( self )
		self.vrot -= self.rot * 0.01
		self.rot += self.vrot

class Rays( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
		self.sprite = getsprite( 'rays%d.png'%random.randint( 1, 2 ) )#, blend_dest = GL_ONE )
		self.rot = 0
		self.vrot = 1
	def draw( self ):
		glTranslatef( 0, 20 - 0.5 * self.rot, 0 )
		self.sprite.opacity = 128 + self.rot * 5
		self.sprite.draw()
	def think( self ):
		removeifoutside( self )
		self.vrot -= self.rot * 0.01
		self.rot += self.vrot

class Nothing( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
	def draw( self ):
		pass
	def think( self ):
		removeifoutside( self )

def removeifoutside( self ):
	if self not in controller.objects:
		return
	if self.x < controller.camx - WIDTH:
		controller.objects.remove( self )
	elif self.x > controller.camx + WIDTH * 2:
		controller.objects.remove( self )
	elif self.y < -40:
		controller.objects.remove( self )

def addobj( obj ):
	controller.objects.append( obj )

def removeobj( obj ):
	if obj in controller.objects:
		controller.objects.remove( obj )

def possiblepirate( x, y ):
	time = float( controller.time ) / FPS
	if HARD_MODE:
		time += 1000
	if random.random() < 0.2 and time > 30:
		addobj( Pirate( x, y ) )
	return Nothing( x, y )

def possibleblock( x, y ):
	time = float( controller.time ) / FPS
	if HARD_MODE:
		time += 1000
	if random.random() < 0.1 and random.random() < (time - 30) / 60:
		addobj( Minotaur( x, y + random.random() * 400 - 200 ) )
	elif random.random() < 0.2 and random.random() < (time - 60) / 120:
		addobj( Squid( x, y + random.random() * 400 - 200 ) )
	elif random.random() < 0.3 and random.random() < (time - 90) / 180:
		addobj( Medusa( x, y + random.random() * 400 - 200 ) )
	return Nothing( x, y )

class Pirate( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
		self.sprite = getsprite( 'pirate.png' )
		self.arrow = getsprite( 'arrow.png' )
		self.tentacles_can_grab = True
		self.armed = 0
		self.rot = 0
		self.vrot = 1
	def draw( self ):
		with gletools.Matrix:
			glScalef( 0.8, 0.8, 1 )
			glRotatef( self.rot, 0, 0, 1 )
			self.sprite.draw()
		if self.armed > 0:
			with gletools.Matrix:
				glScalef( self.armed, self.armed, 1 )
				rot = -90 + math.atan2( controller.icarus.y - self.y, controller.icarus.x - self.x ) * 180 / math.pi
				glRotatef( rot, 0, 0, 1 )
				self.arrow.draw()
	def think( self ):
		removeifoutside( self )
		self.vrot -= self.rot * 0.01
		self.rot += self.vrot
		self.armed += 0.01
		if self.armed >= 1:
			playsound( 'arrow.mp3' )
			addobj( Arrow( self.x, self.y ) )
			self.armed = -1

class Arrow( object ):
	def __init__( self, x, y ):
		self.x, self.y = x, y
		self.sprite = getsprite( 'arrow.png' )
		self.vx = controller.icarus.x - x
		self.vy = controller.icarus.y - y
		l = math.sqrt( self.vx ** 2 + self.vy ** 2 )
		spd = 10
		self.vx *= spd / l
		self.vy *= spd / l
	def gethit( self, target ):
		playsound( 'hit.mp3' )
		removeobj( self )
		self.sprite = getsprite( 'arrow-headless.png' )
		return self
	def draw( self ):
		rot = -90 + math.atan2( self.vy, self.vx ) * 180 / math.pi
		glRotatef( rot, 0, 0, 1 )
		self.sprite.draw()
	def think( self ):
		if self.y < -30:
			removeobj( self )
		self.vy -= controller.gravity
		self.x += self.vx
		self.y += self.vy

class Playing( Controller ):
	def __init__( self ):
		self.gravity = 0.1
		self.camx = 0
		self.objects = []
		self.clouds = [Cloud( WIDTH * 2 ) for i in range( 5 )]
		self.icarus = Icarus( WIDTH + 300, 500 )
		self.objects.append( self.icarus )
		self.time = 0
		self.message = pyglet.text.Label( '', x = WIDTH / 2, y = HEIGHT / 2, bold = True, font_size = 40, anchor_x = 'center' )
		cumultime = [1]
		def waitfor( time ):
			before = cumultime[0]
			cumultime[0] += time * FPS
			return before
		self.messages = [
			(waitfor(0.5), ''),
			(waitfor(3), 'Leaving Crete'),
			(waitfor(1.5), ''),
			(waitfor(3), 'Use cursor keys to move!'),
			(waitfor(1), ''),
			(waitfor(3), 'Experiment for a while!'),
			(waitfor(1), ''),
			(waitfor(3), 'Your goal is to...'),
#			(waitfor(1), ''),
#			(waitfor(3), '...survive for 5 minutes...'),
			(waitfor(1), ''),
			(waitfor(3), '...reach Sicily (to the right)...'),
			(waitfor(1), ''),
			(waitfor(3), '...and kill at least 10 monsters!'),
			(waitfor(1), ''),
			(waitfor(3), 'The land of freedom awaits!'),
			(waitfor(1), ''),
		]
		try:
			assert BLUR_ON
			self.fbo = gletools.Framebuffer( gletools.Texture( WIDTH, HEIGHT, data = [100,100,100,255]*WIDTH*HEIGHT ) )
			self.fbo1 = gletools.Framebuffer( gletools.Texture( WIDTH, HEIGHT, data = [100,100,100,255]*WIDTH*HEIGHT ) )
			self.projection = gletools.Projection( 0, 0, WIDTH, HEIGHT )
			glEnable( GL_BLEND )
			glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA )
			glEnable( GL_COLOR_MATERIAL )
		except:
			self.fbo = None
	def draw( self ):
		glLoadIdentity()
		if self.fbo is not None:
			with self.fbo1:
				self.drawscene()
			with self.fbo.textures[0]:
				gletools.quad( 0, 0, WIDTH, HEIGHT )
			glColor4f( 1, 1, 1, 0.3 )
			with self.fbo1.textures[0]:
				gletools.quad( 0, 0, WIDTH, HEIGHT )
			glColor4f( 1, 1, 1, 1 )
			with self.fbo:
				glColor4f( 1, 1, 1, 0.04 )
				with self.fbo1.textures[0]:
					gletools.quad( 0, 0, WIDTH, HEIGHT )
				glColor4f( 1, 1, 1, 1 )
		else:
			self.drawscene()
		self.drawmessage()
	def drawmessage( self ):
		self.message.x += 2
		self.message.y -= 4
		self.message.color = (0, 0, 0, 100)
		self.message.draw()
		self.message.x -= 2
		self.message.y += 4
		self.message.color = (255, 255, 255, 255)
		self.message.draw()
	def setmessage( self ):
		for time, text in self.messages:
			if time == self.time:
				self.message.text = text
	def drawscene( self ):
		glClearColor( 0x6e / 255.0, 0xc3 / 255.0, 0xff / 255.0, 1 )
		window.clear()
		with gletools.Matrix:
			glLoadIdentity()
			for c in self.clouds:
				c.draw()
			glTranslatef( -self.camx, 0, 0 )
			for obj in self.objects:
				with gletools.Matrix:
					glTranslatef( obj.x, obj.y, 0 )
					obj.draw()
			for obj in self.objects:
				if hasattr( obj, 'draw2' ):
					with gletools.Matrix:
						glTranslatef( obj.x, obj.y, 0 )
						obj.draw2()
	def think( self ):
		self.time += 1
		self.setmessage()
		for cloud in self.clouds:
			cloud.think()
		for obj in self.objects[:]:
			obj.think()
		def getobj( x, y ):
			for obj in self.objects:
				if obj.x == x and obj.y == y:
					return obj
			else:
				return None
		def assertobjs( clazz, y, step ):
			offset = int( self.camx ) / step
			for i in range( offset, offset + (WIDTH + 2*step + 300) / step ):
				x = i * step
				if not getobj( x, y ):
					obj = clazz( x, y )
					self.objects.append( obj )
		assertobjs( Water, 40, 100 )
		assertobjs( Rays, HEIGHT - 40, 100 )
		assertobjs( possiblepirate, 90, 100 )
		assertobjs( possibleblock, 400, 300 )
	def on_key_press( self, symbol, modifiers ):
		for obj in self.objects[:]:
			if hasattr( obj, 'on_key_press' ):
				obj.on_key_press( symbol, modifiers )
	def on_key_release( self, symbol, modifiers ):
		for obj in self.objects[:]:
			if hasattr( obj, 'on_key_release' ):
				obj.on_key_release( symbol, modifiers )

def main():
	if MUSIC:
		music = pyglet.media.Player()
		music.volume = MUSIC
		music.queue( pyglet.resource.media( 'icarus.mp3' ) )
		music.play()
		music.eos_action = music.EOS_LOOP
	global window, keys
	global controller
	window = pyglet.window.Window( vsync = False, width = WIDTH, height = HEIGHT )
	controller = Playing()
	keys = {}
	clock = pyglet.clock.Clock()
	@window.event
	def on_draw():
		controller.draw()
		window.invalid = False
	@window.event
	def on_mouse_motion( *args ):
		controller.on_mouse_motion( *args )
	@window.event
	def on_mouse_press( *args ):
		controller.on_mouse_press( *args )
	@window.event
	def on_mouse_release( *args ):
		controller.on_mouse_release( *args )
	@window.event
	def on_mouse_drag( *args ):
		controller.on_mouse_drag( *args )
	@window.event
	def on_key_press( *args ):
		keys[args[0]] = True
		controller.on_key_press( *args )
	@window.event
	def on_key_release( *args ):
		del keys[args[0]]
		controller.on_key_release( *args )
	def think( dt ):
		clock.tick()
		window.set_caption( 'FPS: %.1f'%clock.get_fps() )
		controller.think()
		window.invalid = True
	pyglet.clock.schedule_interval( think, 1.0/FPS )
	pyglet.app.run()

